@@ -8,7 +8,7 @@ from .cloud import KodexaCloudSession, KodexaCloudPipeline, KodexaCloudService
 from .connectors import FolderConnector, FileHandleConnector, UrlConnector, registered_connectors, get_connectors, \
     get_connector, add_connector, get_source
 from .extractors import *
-from .model import DocumentMetadata, ContentNode, ContentFeature, DocumentRender, DefaultDocumentRender, SourceMetadata, \
+from .model import DocumentMetadata, ContentNode, ContentFeature, DocumentRender, SourceMetadata, \
     Document
 from .pipeline import Pipeline, PipelineStatistics, PipelineContext
 from .sinks import InMemoryDocumentSink
