@@ -1,4 +1,5 @@
 import json
+import logging
 import os
 import time
 
@@ -10,6 +11,8 @@ from kodexa.model import Document
 from kodexa.pipeline import PipelineContext
 from kodexa.stores import TableDataStore
 
+logger = logging.getLogger('kodexa.cloud')
+
 
 class KodexaCloudSession:
     """
@@ -45,7 +48,6 @@ class KodexaCloudSession:
                           data=data,
                           headers={"x-access-token": self.access_token}, files=files)
         execution = Dict(json.loads(r.text))
-        print(execution)
         return execution
 
     def wait_for_execution(self, execution):
@@ -56,25 +58,21 @@ class KodexaCloudSession:
                              headers={"x-access-token": self.access_token})
             execution = Dict(json.loads(r.text))
             if status != execution.status:
-                print(f"Status changed from {status} -> {execution.status}")
+                logger.info(f"Status changed from {status} -> {execution.status}")
                 status = execution.status
             time.sleep(1)
 
         if status == "FAILED":
-            print(execution)
+            logger.error("Failed to execution in session")
+            logger.exception(execution)
             raise Exception("Processing has failed")
 
         return execution
 
     def get_output_document(self, execution):
-        final_reference = None
-        for document_reference in execution.documentReferences:
-            if document_reference.referenceType == 'OUTPUT':
-                final_reference = document_reference
-
-        if final_reference:
+        if execution.outputId:
             doc = requests.get(
-                f"{self.cloud_url}/api/sessions/{self.cloud_session.id}/executions/{execution.id}/documents/{final_reference.cloudDocument.id}",
+                f"{self.cloud_url}/api/sessions/{self.cloud_session.id}/executions/{execution.id}/objects/{execution.outputId}",
                 headers={"x-access-token": self.access_token})
             return Document.from_msgpack(doc.content)
         else:
@@ -97,8 +95,12 @@ class KodexaCloudPipeline:
     Allow you to interact with a pipeline that has been deployed in the Kodexa Cloud
     """
 
-    def __init__(self, slug, version=None, attach_source=True, options={}, auth=[],
+    def __init__(self, slug, version=None, attach_source=True, options=None, auth=None,
                  cloud_url='https://cloud.kodexa.com', access_token=None):
+        if auth is None:
+            auth = []
+        if options is None:
+            options = {}
         self.slug = slug
         self.version = version
         self.attach_source = attach_source
@@ -111,8 +113,11 @@ class KodexaCloudPipeline:
         cloud_session = KodexaCloudSession("pipeline", self.slug, self.access_token, self.cloud_url)
         cloud_session.start()
 
-        connector = FileHandleConnector(input)
-        document = connector.__next__()
+        if isinstance(input, Document):
+            document = input
+        else:
+            connector = FileHandleConnector(input)
+            document = connector.__next__()
         context = PipelineContext()
         execution = cloud_session.execute_service(document, self.options, self.attach_source)
         execution = cloud_session.wait_for_execution(execution)
